// Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.

/*
 * Uses the YouTube IFrame Player API
 * See: https://developers.google.com/youtube/player_parameters?playerVersion=HTML5
 *
 * The majority of this is copied from the example in that guide.
 */

var cloud9KidsPlayer = {}

cloud9KidsPlayer.hasLoadedFirstVideo = false;

/*
 -1 (unstarted)
  0 (ended)
  1 (playing)
  2 (paused)
  3 (buffering)
  5 (video cued)
 */

var stateStrings = [];
/******* API callbacks *******/

// NOTE: all methods that are passed to the events handler will
// not run under the object, and therefore cannot use "this"

// The API will call this function when the video player is ready.
cloud9KidsPlayer.onPlayerReady = function(event) {
    window.console.debug("Cloud9Kids YouTube player:: " + "Player ready");
    androidVideoPlayerProxy.onPlayerReady();
    cloud9KidsPlayer.onTitleAvailable();
}

// The API will call this function on error
cloud9KidsPlayer.onPlayerError = function(event) {
    window.console.debug("Cloud9Kids YouTube player:: " + "Video player error: " + event.data);
    androidVideoPlayerProxy.onPlayerError(event.data);
}

// The API calls this function when the player's state changes.
cloud9KidsPlayer.onPlayerStateChange = function(event) {
    window.console.debug("Cloud9Kids YouTube player:: " + "onPlayerStateChange :" + event.data);
    androidVideoPlayerProxy.onPlayerStateChange(stateStrings[event.data]);
}

// Callback function to get the video title
cloud9KidsPlayer.onTitleAvailable = function() {
    var title = "";

    try {
        if (cloud9KidsPlayer.player != null) {
            var videoData = cloud9KidsPlayer.player.getVideoData();
            title = videoData['title'];
        }
        window.console.debug("Cloud9Kids YouTube player:: " + "Returning video title: " + title);
        androidVideoPlayerProxy.onTitleAvailable(title);
    } catch(err) {
        // if getVideoData() doesn't work, try to get title from OEMBED endpoint
        androidVideoPlayerProxy.getTitle(cloud9KidsPlayer.videoId);
    }
}

// Creates an iframe with the given videoId
cloud9KidsPlayer.java_playVideoId = function(id) {
    cloud9KidsPlayer.player = new YT.Player('player', {
        'width': '100%',
        'videoId': id,
        'playerVars': {
            'cc_load_policy': '0',
            'rel': '0', // Do not show related videos
            'modestbranding': '0', // We want it to be obvious it is YouTube
            'controls': '1', // We can't have custom controls
            'autoplay': '1',
            'html5': '1',
            'showinfo': '0',
            'iv_load_policy': '3', // Disable annotations
        },
        'events': {
            'onReady': cloud9KidsPlayer.onPlayerReady,
            'onStateChange': cloud9KidsPlayer.onPlayerStateChange,
            'onError': cloud9KidsPlayer.onPlayerError
        }
    });

    stateStrings[-1] = "UNSTARTED";
    stateStrings[YT.PlayerState.ENDED]     = "ENDED";
    stateStrings[YT.PlayerState.PLAYING]   = "PLAYING";
    stateStrings[YT.PlayerState.PAUSED]    = "PAUSED";
    stateStrings[YT.PlayerState.BUFFERING] = "BUFFERING";
    stateStrings[YT.PlayerState.CUED]      = "CUED";
}

cloud9KidsPlayer.play = function(id) {
    cloud9KidsPlayer.player.loadVideoById(id, 0, "large");
    cloud9KidsPlayer.onTitleAvailable();
}

cloud9KidsPlayer.java_initializePlayer = function(id) {
    window.console.debug("Cloud9Kids YouTube player:: " + "Initializing for videoId: " + id);
    this.videoId = id;
    if (!cloud9KidsPlayer.hasLoadedFirstVideo) {
        // This should be done after we set the videoId so we don't have the player
        // load without knowing the Id
        var tag = document.createElement('script');
        tag.src = "https://www.youtube.com/iframe_api";
        var firstScriptTag = document.getElementsByTagName('script')[0];
        firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
        cloud9KidsPlayer.hasLoadedFirstVideo = true;
    } else {
        window.console.debug("Cloud9Kids YouTube player:: " + "Player already intialized. Loading new video: " + id);
        cloud9KidsPlayer.play(id);
    }
}

// Helper function for java class to play video
// TODO (rpruden@amazon.com): FREETIME-932: Clean up JS VideoPlayer play/initialize logic
cloud9KidsPlayer.java_playVideo = function() {
    window.console.debug("Cloud9Kids YouTube player:: " + "java_playVideo");
    cloud9KidsPlayer.player.loadVideoById(cloud9KidsPlayer.videoId, 0, "large");
}

// Helper function for java class to pause video
cloud9KidsPlayer.java_pauseVideo = function() {
    if (cloud9KidsPlayer.player) {
        cloud9KidsPlayer.player.pauseVideo();
        window.console.debug("Cloud9Kids YouTube player:: " + "java_pauseVideo");
    }
}

// Helper function for java class to get elapsed time
cloud9KidsPlayer.java_getUpdatedProgress = function() {
    window.console.debug("Cloud9Kids YouTube player:: " + "java_getUpdatedProgress");
    return this.player.getCurrentTime();
}

// Helper function for java class to get video duration
cloud9KidsPlayer.java_getVideoDuration = function() {
    window.console.debug("Cloud9Kids YouTube player:: " + "java_getVideoDuration");
    return this.player.getDuration();
}

// Helper function for java class to stop the video
cloud9KidsPlayer.java_stopVideo = function() {
    window.console.debug("Cloud9Kids YouTube player:: " + "stopVideo()");
    this.player.stopVideo();
}

// Creates an <iframe> (and YouTube player)
// after the API code downloads.
// This needs to be global for the API to call back
function onYouTubeIframeAPIReady() {
    window.console.debug("Cloud9Kids YouTube player:: " + "onYouTubeIframeAPIReady");
    cloud9KidsPlayer.java_playVideoId(cloud9KidsPlayer.videoId);
}


